@@ -1,16 +1,12 @@
 from __future__ import annotations
 
 import sys
-from typing import ClassVar, Dict, Iterable, List, Optional, Type, Union
+from dataclasses import dataclass
+from typing import ClassVar, Iterable, get_args
 
 from markdown_it import MarkdownIt
 from markdown_it.token import Token
 
-if sys.version_info >= (3, 8):
-    from typing import get_args
-else:
-    from typing_extensions import get_args  # pragma: no cover
-
 from rich.table import Table
 
 from . import box
@@ -19,7 +15,6 @@ from ._stack import Stack
 from .console import Console, ConsoleOptions, JustifyMethod, RenderResult
 from .containers import Renderables
 from .jupyter import JupyterMixin
-from .panel import Panel
 from .rule import Rule
 from .segment import Segment
 from .style import Style, StyleStack
@@ -31,7 +26,7 @@ class MarkdownElement:
     new_line: ClassVar[bool] = True
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "MarkdownElement":
+    def create(cls, markdown: Markdown, token: Token) -> MarkdownElement:
         """Factory to create markdown element,
 
         Args:
@@ -43,30 +38,28 @@ class MarkdownElement:
         """
         return cls()
 
-    def on_enter(self, context: "MarkdownContext") -> None:
+    def on_enter(self, context: MarkdownContext) -> None:
         """Called when the node is entered.
 
         Args:
             context (MarkdownContext): The markdown context.
         """
 
-    def on_text(self, context: "MarkdownContext", text: TextType) -> None:
+    def on_text(self, context: MarkdownContext, text: TextType) -> None:
         """Called when text is parsed.
 
         Args:
             context (MarkdownContext): The markdown context.
         """
 
-    def on_leave(self, context: "MarkdownContext") -> None:
+    def on_leave(self, context: MarkdownContext) -> None:
         """Called when the parser leaves the element.
 
         Args:
             context (MarkdownContext): [description]
         """
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         """Called when a child element is closed.
 
         This method allows a parent element to take over rendering of its children.
@@ -81,8 +74,8 @@ class MarkdownElement:
         return True
 
     def __rich_console__(
-        self, console: "Console", options: "ConsoleOptions"
-    ) -> "RenderResult":
+        self, console: Console, options: ConsoleOptions
+    ) -> RenderResult:
         return ()
 
 
@@ -100,14 +93,14 @@ class TextElement(MarkdownElement):
 
     style_name = "none"
 
-    def on_enter(self, context: "MarkdownContext") -> None:
+    def on_enter(self, context: MarkdownContext) -> None:
         self.style = context.enter_style(self.style_name)
         self.text = Text(justify="left")
 
-    def on_text(self, context: "MarkdownContext", text: TextType) -> None:
+    def on_text(self, context: MarkdownContext, text: TextType) -> None:
         self.text.append(text, context.current_style if isinstance(text, str) else None)
 
-    def on_leave(self, context: "MarkdownContext") -> None:
+    def on_leave(self, context: MarkdownContext) -> None:
         context.leave_style()
 
 
@@ -118,7 +111,7 @@ class Paragraph(TextElement):
     justify: JustifyMethod
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "Paragraph":
+    def create(cls, markdown: Markdown, token: Token) -> Paragraph:
         return cls(justify=markdown.justify or "left")
 
     def __init__(self, justify: JustifyMethod) -> None:
@@ -131,14 +124,29 @@ class Paragraph(TextElement):
         yield self.text
 
 
+@dataclass
+class HeadingFormat:
+    justify: JustifyMethod = "left"
+    style: str = ""
+
+
 class Heading(TextElement):
     """A heading."""
 
+    LEVEL_ALIGN: ClassVar[dict[str, JustifyMethod]] = {
+        "h1": "center",
+        "h2": "left",
+        "h3": "left",
+        "h4": "left",
+        "h5": "left",
+        "h6": "left",
+    }
+
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "Heading":
+    def create(cls, markdown: Markdown, token: Token) -> Heading:
         return cls(token.tag)
 
-    def on_enter(self, context: "MarkdownContext") -> None:
+    def on_enter(self, context: MarkdownContext) -> None:
         self.text = Text()
         context.enter_style(self.style_name)
 
@@ -150,20 +158,10 @@ class Heading(TextElement):
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
-        text = self.text
-        text.justify = "center"
-        if self.tag == "h1":
-            # Draw a border around h1s
-            yield Panel(
-                text,
-                box=box.HEAVY,
-                style="markdown.h1.border",
-            )
-        else:
-            # Styled text for h2 and beyond
-            if self.tag == "h2":
-                yield Text("")
-            yield text
+        text = self.text.copy()
+        heading_justify = self.LEVEL_ALIGN.get(self.tag, "left")
+        text.justify = heading_justify
+        yield text
 
 
 class CodeBlock(TextElement):
@@ -172,10 +170,10 @@ class CodeBlock(TextElement):
     style_name = "markdown.code_block"
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "CodeBlock":
+    def create(cls, markdown: Markdown, token: Token) -> CodeBlock:
         node_info = token.info or ""
         lexer_name = node_info.partition(" ")[0]
-        return cls(lexer_name or "default", markdown.code_theme)
+        return cls(lexer_name or "text", markdown.code_theme)
 
     def __init__(self, lexer_name: str, theme: str) -> None:
         self.lexer_name = lexer_name
@@ -199,9 +197,7 @@ class BlockQuote(TextElement):
     def __init__(self) -> None:
         self.elements: Renderables = Renderables()
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         self.elements.append(child)
         return False
 
@@ -228,7 +224,8 @@ class HorizontalRule(MarkdownElement):
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
         style = console.get_style("markdown.hr", default="none")
-        yield Rule(style=style)
+        yield Rule(style=style, characters="-")
+        yield Text()
 
 
 class TableElement(MarkdownElement):
@@ -238,9 +235,7 @@ class TableElement(MarkdownElement):
         self.header: TableHeaderElement | None = None
         self.body: TableBodyElement | None = None
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         if isinstance(child, TableHeaderElement):
             self.header = child
         elif isinstance(child, TableBodyElement):
@@ -252,11 +247,19 @@ class TableElement(MarkdownElement):
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
     ) -> RenderResult:
-        table = Table(box=box.SIMPLE_HEAVY)
+        table = Table(
+            box=box.SIMPLE,
+            pad_edge=False,
+            style="markdown.table.border",
+            show_edge=True,
+            collapse_padding=True,
+        )
 
         if self.header is not None and self.header.row is not None:
             for column in self.header.row.cells:
-                table.add_column(column.content)
+                heading = column.content.copy()
+                heading.stylize("markdown.table.header")
+                table.add_column(heading)
 
         if self.body is not None:
             for row in self.body.rows:
@@ -272,9 +275,7 @@ class TableHeaderElement(MarkdownElement):
     def __init__(self) -> None:
         self.row: TableRowElement | None = None
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         assert isinstance(child, TableRowElement)
         self.row = child
         return False
@@ -286,9 +287,7 @@ class TableBodyElement(MarkdownElement):
     def __init__(self) -> None:
         self.rows: list[TableRowElement] = []
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         assert isinstance(child, TableRowElement)
         self.rows.append(child)
         return False
@@ -298,11 +297,9 @@ class TableRowElement(MarkdownElement):
     """MarkdownElement corresponding to `tr_open` and `tr_close`."""
 
     def __init__(self) -> None:
-        self.cells: List[TableDataElement] = []
+        self.cells: list[TableDataElement] = []
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         assert isinstance(child, TableDataElement)
         self.cells.append(child)
         return False
@@ -313,8 +310,8 @@ class TableDataElement(MarkdownElement):
     and `th_open` and `th_close`."""
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "MarkdownElement":
-        style = str(token.attrs.get("style" "")) or ""
+    def create(cls, markdown: Markdown, token: Token) -> MarkdownElement:
+        style = str(token.attrs.get("style")) or ""
 
         justify: JustifyMethod
         if "text-align:right" in style:
@@ -330,32 +327,28 @@ class TableDataElement(MarkdownElement):
         return cls(justify=justify)
 
     def __init__(self, justify: JustifyMethod) -> None:
-        self.content: TextType = ""
+        self.content: Text = Text("", justify=justify)
         self.justify = justify
 
-    def on_text(self, context: "MarkdownContext", text: TextType) -> None:
-        plain = text.plain if isinstance(text, Text) else text
-        style = text.style if isinstance(text, Text) else ""
-        self.content = Text(
-            plain, justify=self.justify, style=context.style_stack.current
-        )
+    def on_text(self, context: MarkdownContext, text: TextType) -> None:
+        text = Text(text) if isinstance(text, str) else text
+        text.stylize(context.current_style)
+        self.content.append_text(text)
 
 
 class ListElement(MarkdownElement):
     """A list element."""
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "ListElement":
+    def create(cls, markdown: Markdown, token: Token) -> ListElement:
         return cls(token.type, int(token.attrs.get("start", 1)))
 
     def __init__(self, list_type: str, list_start: int | None) -> None:
-        self.items: List[ListItem] = []
+        self.items: list[ListItem] = []
         self.list_type = list_type
         self.list_start = list_start
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         assert isinstance(child, ListItem)
         self.items.append(child)
         return False
@@ -383,9 +376,7 @@ class ListItem(TextElement):
     def __init__(self) -> None:
         self.elements: Renderables = Renderables()
 
-    def on_child_close(
-        self, context: "MarkdownContext", child: "MarkdownElement"
-    ) -> bool:
+    def on_child_close(self, context: MarkdownContext, child: MarkdownElement) -> bool:
         self.elements.append(child)
         return False
 
@@ -421,7 +412,7 @@ class ListItem(TextElement):
 
 class Link(TextElement):
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "MarkdownElement":
+    def create(cls, markdown: Markdown, token: Token) -> MarkdownElement:
         url = token.attrs.get("href", "#")
         return cls(token.content, str(url))
 
@@ -436,7 +427,7 @@ class ImageItem(TextElement):
     new_line = False
 
     @classmethod
-    def create(cls, markdown: "Markdown", token: Token) -> "MarkdownElement":
+    def create(cls, markdown: Markdown, token: Token) -> MarkdownElement:
         """Factory to create markdown element,
 
         Args:
@@ -451,10 +442,10 @@ class ImageItem(TextElement):
     def __init__(self, destination: str, hyperlinks: bool) -> None:
         self.destination = destination
         self.hyperlinks = hyperlinks
-        self.link: Optional[str] = None
+        self.link: str | None = None
         super().__init__()
 
-    def on_enter(self, context: "MarkdownContext") -> None:
+    def on_enter(self, context: MarkdownContext) -> None:
         self.link = context.current_style.link
         self.text = Text(justify="left")
         super().on_enter(context)
@@ -478,7 +469,7 @@ class MarkdownContext:
         console: Console,
         options: ConsoleOptions,
         style: Style,
-        inline_code_lexer: Optional[str] = None,
+        inline_code_lexer: str | None = None,
         inline_code_theme: str = "monokai",
     ) -> None:
         self.console = console
@@ -486,7 +477,7 @@ class MarkdownContext:
         self.style_stack: StyleStack = StyleStack(style)
         self.stack: Stack[MarkdownElement] = Stack()
 
-        self._syntax: Optional[Syntax] = None
+        self._syntax: Syntax | None = None
         if inline_code_lexer is not None:
             self._syntax = Syntax("", inline_code_lexer, theme=inline_code_theme)
 
@@ -506,7 +497,7 @@ class MarkdownContext:
         else:
             self.stack.top.on_text(self, text)
 
-    def enter_style(self, style_name: Union[str, Style]) -> Style:
+    def enter_style(self, style_name: str | Style) -> Style:
         """Enter a style context."""
         style = self.console.get_style(style_name, default="none")
         self.style_stack.push(style)
@@ -523,7 +514,7 @@ class Markdown(JupyterMixin):
 
     Args:
         markup (str): A string containing markdown.
-        code_theme (str, optional): Pygments theme for code blocks. Defaults to "monokai".
+        code_theme (str, optional): Pygments theme for code blocks. Defaults to "monokai". See https://pygments.org/styles/ for code themes.
         justify (JustifyMethod, optional): Justify value for paragraphs. Defaults to None.
         style (Union[str, Style], optional): Optional style to apply to markdown.
         hyperlinks (bool, optional): Enable hyperlinks. Defaults to ``True``.
@@ -533,7 +524,7 @@ class Markdown(JupyterMixin):
             highlighting, or None for no highlighting. Defaults to None.
     """
 
-    elements: ClassVar[Dict[str, Type[MarkdownElement]]] = {
+    elements: ClassVar[dict[str, type[MarkdownElement]]] = {
         "paragraph_open": Paragraph,
         "heading_open": Heading,
         "fence": CodeBlock,
@@ -558,17 +549,17 @@ class Markdown(JupyterMixin):
         self,
         markup: str,
         code_theme: str = "monokai",
-        justify: Optional[JustifyMethod] = None,
-        style: Union[str, Style] = "none",
+        justify: JustifyMethod | None = None,
+        style: str | Style = "none",
         hyperlinks: bool = True,
-        inline_code_lexer: Optional[str] = None,
-        inline_code_theme: Optional[str] = None,
+        inline_code_lexer: str | None = None,
+        inline_code_theme: str | None = None,
     ) -> None:
         parser = MarkdownIt().enable("strikethrough").enable("table")
         self.markup = markup
         self.parsed = parser.parse(markup)
         self.code_theme = code_theme
-        self.justify: Optional[JustifyMethod] = justify
+        self.justify: JustifyMethod | None = justify
         self.style = style
         self.hyperlinks = hyperlinks
         self.inline_code_lexer = inline_code_lexer
@@ -695,7 +686,7 @@ class Markdown(JupyterMixin):
                         and context.stack.top.on_child_close(context, element)
                     )
                     if should_render:
-                        if new_line:
+                        if new_line and node_type != "inline":
                             yield _new_line_segment
                         yield from console.render(element, context.options)
 
@@ -774,7 +765,7 @@ if __name__ == "__main__":  # pragma: no cover
     if args.path == "-":
         markdown_body = sys.stdin.read()
     else:
-        with open(args.path, "rt", encoding="utf-8") as markdown_file:
+        with open(args.path, encoding="utf-8") as markdown_file:
             markdown_body = markdown_file.read()
 
     markdown = Markdown(
